<?php
/**
 * pragmaMx - Recipe Module
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.12 $
 * $Author: tora60 $
 * $Date: 2011/11/20 11:39:08 $
 *
 * this module based on: PHP-NUKE Recipe Module
 * Copyright (c) 2003 by Sean Pluard , http://heartstorm.org
 * modified by jubilee (http://www.marx-city.de) and kochloeffel
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * pmxRecipe
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2011
 * @version $Id: class.recipe.php,v 1.12 2011/11/20 11:39:08 tora60 Exp $
 * @access public
 */
class pmxRecipe {
    protected $_module_name = '';
    protected $_dbprefix = '';
    protected $_dbtable = '';

    protected $_messages = array();
    protected $_errors = array();

    protected $_config = array();
    protected $_template;
    protected $_tabs;

    protected $_is_admin = false;
    protected $_is_autor = false;
    protected $_adminmode = false; // es luft ber die admin.php

    protected $_defaults = array();
    protected $_user = array();

    /**
     * pmxRecipe::__construct()
     */
    protected function __construct()
    {
        $this->_module_name = basename(dirname(dirname(__FILE__)));
        mxGetLangfile($this->_module_name);

        include(dirname(__FILE__) . DS . 'settings.php');
        $this->_config = $recipe_config;

        switch (true) {
            case MX_IS_USER:
                $userinfo = mxGetUserData();
                $this->_user = array(/* Userdaten */
                    'uname' => $userinfo['uname'],
                    'email' => $userinfo['email'],
                    );
                break;
            case MX_IS_ADMIN:
                $userinfo = mxGetAdminData();
                if ($userinfo['user_uid']) {
                    /* Userdaten */
                    $userinfo = mxGetUserDataFromUid($userinfo['user_uid']);
                    $this->_user = array(/* Userdaten aus Admindaten */
                        'uname' => $userinfo['uname'],
                        'email' => $userinfo['email'],
                        );
                } else {
                    $this->_user = array(/* Userdaten */
                        'uname' => $userinfo['aid'],
                        'email' => $userinfo['email'],
                        );
                }

                break;
            default:
                $this->_user = array(/* Userdaten */
                    'uname' => '',
                    'email' => '',
                    );
        }

        /* Administratorgruppen die das Modul administrieren drfen */
        $this->_is_admin = false;
        switch (true) {
            case MX_IS_SYSADMIN:
                $this->_is_admin = true;
                break;

            case !isset($this->_config['adminaccess']):
            case !is_array($this->_config['adminaccess']):
                break;

            case MX_IS_ADMIN:
                foreach ($this->_config['adminaccess'] as $radmin) {
                    if (mxGetAdminPref($radmin)) {
                        $this->_is_admin = true;
                        break;
                    }
                }
        }

        /* wer darf Rezepte eingeben, seine editieren und Bilder dazu hochladen?
         * - nur Admins = false
         * - Admins und alle angemeldeten User = true
         * - Admins und bestimmte Usergruppen = Array der Usergruppen-ID's z.B. array(3, 12, 8)
         */
        switch (true) {
            case empty($this->_config['writeaccess']):
                $this->_is_autor = $this->_is_admin;
                break;

            case $this->_config['writeaccess'] === true:
                $this->_is_autor = ($this->_is_admin || MX_IS_USER);
                break;

            case is_array($this->_config['writeaccess']) && isset($userinfo['user_ingroup']):
                if (in_array($userinfo['user_ingroup'], $this->_config['writeaccess'])) {
                    $this->_is_autor = true;
                    break;
                }

            default:
                $this->_is_autor = $this->_is_admin;
        }

        /* Datenbankkonfiguration */
        $this->_dbprefix = $GLOBALS['prefix'];
        $this->_dbtable_rec = $this->_dbprefix . '_recipes';
        $this->_dbtable_cat = $this->_dbprefix . '_recipes_categories';

        $this->_defaults = array(/* Standardwerte */
            'id' => 0,
            'recipename' => '',
            'description' => '',
            'ingredients' => '',
            'directions' => '',
            'nutrition' => '',
            'tipps' => '',
            'cid' => 0,
            'source' => '',
            'preptime' => 0,
            'cooktime' => 0,
            'difficulty' => '',
            'persons' => '',
            'contributer' => $this->_user['uname'],
            'photo' => '',
            'submittime' => date('Y-m-d H:i:s'),
            'catname' => '',
            );

        /* modulspezifisches Stylesheet einbinden */
        pmxHeader::add_style(PMX_MODULES_PATH . $this->_module_name . '/style/style.css');

        /* Template initialisieren */
        $this->_template = load_class('Template');
        $this->_template->init_path(__FILE__);
        $this->_template->assign($this->_config);
        $this->_template->assign('parent', $this);
        $this->_template->assign('is_admin', $this->_is_admin);
        $this->_template->assign('module_name', $this->_module_name);
    }

    public function display_entry($data)
    {
        // $this->_template->assign(compact('row', 'reporter'));
        $this->_template->assign($data);
        return $this->_template->fetch('view.html');
    }

    public function recipe_form($row)
    {
        if ($row['id']) {
            /* edit existed */
            $action = ($this->_adminmode) ? 'admin.php?op=recipe/edit/send' : 'modules.php?name=' . $this->_module_name . '&amp;op=edit/send';
            $caption = _REC_EDITRECIPEFORM;
            $submit = _SAVE;
        } else {
            /* addnew */
            $action = ($this->_adminmode) ? 'admin.php?op=recipe/add/send' : 'modules.php?name=' . $this->_module_name . '&amp;op=add/send';
            $caption = _REC_NEWRECIPEFORM;
            $submit = _REC_SUBMITTHISRECIPE;
        }

        $cats = array();
        if ($this->_is_admin) {
            $sel = (empty($row['cid'])) ? ' selected="selected" class="current"' : '';
            $cats[] = "<option value='0'$sel> -- " . _INACTIVE . " -- </option>";
        }

        if ($row['submittime'] && ($tmpdate = strtotime($row['submittime']))) {
            $row['submittime'] = strftime(_DATEPICKER, $tmpdate);
        }

        /* Schleifchen :-)  */
        $sql = "SELECT cid, catname FROM `$this->_dbtable_cat` ORDER BY catname";
        $result = sql_query($sql);
        while (list($cid, $catname) = sql_fetch_row($result)) {
            $sel = ($cid == $row['cid']) ? ' selected="selected" class="current"' : '';
            $cats[] = "<option value='$cid'$sel>$catname</option>";
        }
        $catoptions = implode("\n", $cats);

        $difficulties = $this->recipe_difficulties();
        $diffs = array();
        foreach ($difficulties as $value => $text) {
            switch (true) {
                case $row['difficulty'] == $value:
                case $row['difficulty'] == $text:
                    $sel = ' selected="selected" class="current"';
                    break;
                default:
                    $sel = '';
            }
            $diffs[] = "<option value=\"$value\"$sel>$text</option>";
        }
        $diffoptions = implode("\n", $diffs);

        $wysiwyg = load_class('Textarea');
        $wysiwyg->setWysiwyg($this->use_wysiwyg);
        /* Textarea Felder */
        $fields = array('description' => _REC_DESCRIPTION,
            'ingredients' => _REC_INGREDIENTS,
            'directions' => _REC_DIRECTIONS,
            'nutrition' => _REC_NUTRITIONINFORMATION,
            'tipps' => _REC_TIPP,
            );

        $this->_template->assign(compact('row', 'size', 'caption', 'submit', 'action', 'wysiwyg', 'fields', 'diffoptions', 'catoptions'));
        $this->_template->assign('adminmode', $this->_adminmode);
        return $this->_template->fetch('form.html');
    }

    public function list_entries($rowset)
    {
        foreach ($rowset as $key => $row) {
            if (!$row['description']) {
                $row['description'] = strip_tags($row['directions']);
            }
            if (intval($this->strip_desc)) {
                $row['description'] = mxCutString(strip_tags($row['description']), intval($this->strip_desc));
            }
            $row['reporter'] = $this->has_edit_access($row['contributer']);

            $rowset[$key] = $row;
        }

        $this->_template->assign('rowset', $rowset);
        $this->_template->assign('is_admin', $this->_is_admin);
        return $this->_template->fetch('list_entries.html');
    }

    public function has_edit_access($check_name)
    {
        switch (true) {
            case !$check_name:
            case !$this->_user['uname']:
            case !$this->_is_autor:
                return false;
            case $this->_is_admin:
            case $this->_user['uname'] == $check_name:
                return true;
            default:
                return false;
        }
    }

    public function has_write_access()
    {
        return $this->_is_autor;
    }

    public function recipe_get_data($id)
    {
        $sql = "SELECT rec.*, cat.catname
                FROM `$this->_dbtable_rec` AS rec
                LEFT JOIN `$this->_dbtable_cat` AS cat
                ON rec.cid = cat.cid
                WHERE id=" . intval($id);

        $result = sql_query($sql);
        $row = sql_fetch_assoc($result);
        if (!$row) {
            return $this->_defaults;
        }
        return array_merge($this->_defaults, $row);
    }

    public function recipe_get_data_for_view($id)
    {
        $row = $this->recipe_get_data($id);

        switch (true) {
            case !$id:
            case !$row['id']:
            case strtotime($row['submittime']) > time():
                $this->_errors[] = _REC_NOTEXIST;
                $row = $this->_defaults;
                break;
            default:
                $row['submittime'] = formatTimestamp($row['submittime'], _DATESTRING);
                $row['preptime'] = $this->min2string($row['preptime']);
                $row['cooktime'] = $this->min2string($row['cooktime']);
                $row['difficulty'] = $this->recipe_difficulties($row['difficulty']);

                if (!$row['contributer'] || $row['contributer'] == "Guest" || $row['contributer'] == $GLOBALS['anonymous']) {
                    $row['contributer'] = $GLOBALS['anonymous'];
                }
        }

        return $row;
    }

    public function alpha()
    {
        $alphabet = $this->get_alphabet();

        $result = sql_query("SELECT DISTINCT UPPER(LEFT(recipename, 1)) FROM `$this->_dbtable_rec`");
        while (list($letter) = sql_fetch_row($result)) {
            $titles[$letter] = $letter;
            // verwendete (Sonder)Zeichen ans Alphabet anfgen
            $alphabet[] = $letter;
        }
        $alphabet = array_unique($alphabet);

        $minletters = 12; // min 12 Buchstaben
        $factor = (count($alphabet) % $minletters) + 1 ;
        $i = 0 - $factor;
        foreach ($alphabet as $ltr) {
            $i++;
            if ($i == $minletters) {
                $trenner = '</b> <b class="nowrap">';
                $i = 0;
            } else {
                $trenner = '';
            }
            if (isset($titles[$ltr])) {
                $x[] = $trenner . '<a href="modules.php?name=' . $this->_module_name . '&amp;letter=' . urlencode($ltr) . '">' . $ltr . '</a>';
            } else {
                $x[] = $trenner . '<span>' . $ltr . '</span>';
            }
        }

        return '<div class="pagination"><b class="nowrap">' . implode('', $x) . '</b></div>';
    }

    public function get_alphabet()
    {
        $alphabet = array();

        for ($i = 65; $i <= 90; $i++) {
            $alphabet[] = chr($i);;
        }

        /* Sonderzeichen aus Sprachdatei, aber nur die, die auch als Gross-Zeichen existieren, also nicht das  */
        $uppers = str_split(strtoupper(_SPECIALCHARS));
        $lowers = str_split(strtolower(_SPECIALCHARS));
        $more = array_unique(array_diff($uppers, $lowers));
        $alphabet = array_merge($alphabet, $more);
        return $alphabet;
    }

    public function recipe_difficulties($difficulty = false)
    {
        $difficulties = array(/* Schwierigkeit bersetzen */
            "leicht" => _REC_STATEASY,
            "moderat" => _REC_STATMODERATE,
            "schwer" => _REC_STATHARD,
            "sehr schwer" => _REC_STATDIFFICULT,
            );

        switch (true) {
            case $difficulty === false:
                return $difficulties;
            case $difficulty && isset($difficulties[$difficulty]):
                return $difficulties[$difficulty];
            default:
                return $difficulty;
        }
    }

    protected function del_send_save($id, $row)
    {
        $sql = "DELETE FROM `$this->_dbtable_rec` WHERE id=" . intval($id);
        $result = sql_query($sql);
        if ($result) {
            $this->delete_photo($row);
        }

        $this->_messages[] = _REC_THERECIPE . " <em>" . $row['recipename'] . "</em> " . _REC_HASBEENDELETED;

        return true;
    }

    protected function edit_send_save($data)
    {
        if (!$this->check_recipe($data)) {
            return false;
        }

        if ($data['deletephoto']) {
            $this->delete_photo($data);
            $data['photo'] = '';
        }

        $fname = $this->handle_upload($data);

        extract($data, EXTR_SKIP);

        $sql = "UPDATE `$this->_dbtable_rec` SET
                recipename='" . mxAddSlashesForSQL($recipename) . "',
                description='" . mxAddSlashesForSQL($description) . "',
                ingredients='" . mxAddSlashesForSQL($ingredients) . "',
                directions='" . mxAddSlashesForSQL($directions) . "',
                nutrition='" . mxAddSlashesForSQL($nutrition) . "',
                tipps='" . mxAddSlashesForSQL($tipps) . "',
                cid=" . intval($cid) . ",
                source='" . mxAddSlashesForSQL($source) . "',
                preptime=" . intval($preptime) . ",
                cooktime=" . intval($cooktime) . ",
                difficulty='" . mxAddSlashesForSQL($difficulty) . "',
                persons=" . intval($persons) . ",
                photo='" . mxAddSlashesForSQL($fname) . "',
                submittime='" . mxAddSlashesForSQL($data['submittime']) . "'
                WHERE id=" . intval($id);
        $result = sql_query($sql);

        $this->_messages[] = _REC_THERECIPE . " <em>" . $data['recipename'] . "</em> " . _REC_HASBEENUPDATED;

        return true;
    }

    protected function add_send_save($data)
    {
        if (!$this->check_recipe($data)) {
            return false;
        }

        $sql = "INSERT INTO `$this->_dbtable_rec` SET
                recipename='" . mxAddSlashesForSQL($data['recipename']) . "',
                description='" . mxAddSlashesForSQL($data['description']) . "',
                ingredients='" . mxAddSlashesForSQL($data['ingredients']) . "',
                directions='" . mxAddSlashesForSQL($data['directions']) . "',
                nutrition='" . mxAddSlashesForSQL($data['nutrition']) . "',
                tipps='" . mxAddSlashesForSQL($data['tipps']) . "',
                cid=" . intval($data['cid']) . ",
                source='" . mxAddSlashesForSQL($data['source']) . "',
                preptime=" . intval($data['preptime']) . ",
                cooktime=" . intval($data['cooktime']) . ",
                difficulty='" . mxAddSlashesForSQL($data['difficulty']) . "',
                persons=" . intval($data['persons']) . ",
                contributer='" . mxAddSlashesForSQL($data['contributer']) . "',
                photo='',
                submittime='" . mxAddSlashesForSQL($data['submittime']) . "'";

        $result = sql_query($sql);

        if ($result) {
            $data['id'] = sql_insert_id();
            $fname = $this->handle_upload($data);

            $sql = "UPDATE `$this->_dbtable_rec` SET
                photo='" . mxAddSlashesForSQL($fname) . "'
                WHERE id=" . intval($data['id']);
            $result = sql_query($sql);
            $this->_messages[] = _REC_THERECIPE . " <em>" . $data['recipename'] . "</em> " . _REC_HASBEENADDED;
        } else {
            $this->_errors[] = _REC_THERECIPE . " <em>" . $data['recipename'] . "</em> " . _REC_HASBEENADDEDNOT;
        }

        return $data['id'];
    }

    protected function check_recipe($data)
    {
        if (empty($data['recipename'])) {
            $this->_errors[] = _REC_MUSTNAME;
        }
        if (empty($data['ingredients'])) {
            $this->_errors[] = _REC_MUSTINGREDIENTS;
        }
        if (empty($data['directions'])) {
            $this->_errors[] = _REC_MUSTDIRECTIONS;
        }

        return !$this->_errors;
    }

    protected function handle_upload($row)
    {
        $handle = load_class('Upload', $_FILES['userfile']);

        switch (true) {
            case !$handle->uploaded:
            case !MX_IS_USER && !$this->_is_admin:
            case !$this->has_edit_access($row['contributer']):
                return (($row['photo']) ? $row['photo'] : '');
            case !$this->photomaxfilesize:
            case !is_writeable(realpath($this->photopath)):
                return '';
        }

        $destination = realpath($this->photopath);

        $filename = $this->get_photoname($row);

        foreach ($this->photosizes as $suffix => $width) {
            /* Dateiname... */
            $handle->file_new_name_body = $filename;
            $handle->file_name_body_add = '-' . $suffix;
            /* Punkte im Dateinamen ermoeglichen */
            $handle->file_safe_name = true;
            /* automatisches umbenennen unterdruecken */
            $handle->file_auto_rename = false;
            /* existierende Dateien einfach ueberschreiben */
            $handle->file_overwrite = true;

            /* nur Bilder zulassen */
            $handle->allowed = array('image/*');
            /* immer zu jpg konvertieren */
            $handle->image_convert = 'jpg';

            /* Seitenverhltnis beibehalten */
            $handle->image_ratio_no_zoom_in = true;
            $handle->image_ratio_y = true;

            /* Bildgroesse einstellen */
            $handle->image_resize = true;
            $handle->image_x = intval($width);

            $handle->jpeg_size = $this->photomaxfilesize;

            /* Bild bearbeiten u. speichern */
            $handle->process($destination);

            if ($handle->error) {
                break;
            }
        }

        if (!$handle->error) {
            return $filename;
        }

        echo '<p class="error">' . trim($handle->error, ' .,') . ', ' . _REC_PHOTONOTATTACHED . '</p>';
        return '';
    }

    protected function delete_photo($row)
    {
        $destination = realpath($this->photopath);

        $name = $this->get_photoname($row);

        $files = (array)glob($destination . DS . $name . '*', GLOB_NOSORT);
        if ($row['photo']) {
            /* alter Typ, vor v3.0 */
            $files2 = (array)glob($destination . DS . $row['photo'] . '*', GLOB_NOSORT);
            $files = array_merge($files, $files2);
        }

        foreach ($files as $filename) {
            if (is_file($filename)) {
                unlink($filename);
            }
        }
    }

    public function min2string($minutes)
    {
        $minutes = intval($minutes);

        if (!$minutes) {
            return '';
        }

        $hours = floor($minutes / 60);
        $minutes = ($minutes % 60);

        switch (true) {
            case $hours > 0 && $minutes > 0:
                return sprintf(_REC_FORMATMINUTES_HM, $hours, $minutes);
            case $hours > 0:
                return sprintf(_REC_FORMATMINUTES_H, $hours);
            case $minutes > 0:
                return sprintf(_REC_FORMATMINUTES_M, $minutes);
            default:
                return '';
        }
    }

    public function get_photoname($row)
    {
        $filename = sprintf('recipe-%08d', $row['id']);
        // $filename = preg_replace('#[^a-zA-Z0-9_-]#', '_', $filename);
        // $filename = preg_replace('#_+#', '_', $filename);
        return $filename;
    }

    public function get_photo_imagetag($row, $size = 'small', $class = '')
    {
        $destination = realpath($this->photopath);

        if ($class) {
            $class = ' class="' . $class . '"';
        }

        switch (true) {
            case !$row['photo']:
            // case !$this->photomaxfilesize:
            case !($destination = realpath($this->photopath)):
                return '';
            case $size == 'mini' && is_file($destination . DS . $row['photo']):
                /* alter Typ, vor v3.0 */
                $prettypath = $this->photopath . $row['photo'];
                $imagepath = $this->photopath . $row['photo'];
                break;
            case is_file($destination . DS . $row['photo']):
                /* alter Typ, vor v3.0 */
                $sizes = getimagesize($destination . DS . $row['photo']);
                switch (true) {
                    case !$sizes:
                        return '';
                    case $sizes[0] > $this->photosizes['small']:
                    case $sizes[1] > $this->photosizes['small']:
                        // wenn grsseres Bild, auch als lightbox zeigen
                        $prettypath = $this->photopath . $row['photo'];
                        break;
                    default:
                        $prettypath = false;
                }
                $imagepath = $this->photopath . $row['photo'];
                break;
            case $size == 'big';
                $prettypath = false;
                $imagepath = $this->photopath . $row['photo'] . '-big.jpg';
                break;
            case $size == 'mini';
                $prettypath = $this->photopath . $row['photo'] . '-big.jpg';
                $imagepath = $this->photopath . $row['photo'] . '-mini.jpg';
                break;
            case $size == 'small';
            default:
                $size = 'small';
                $prettypath = $this->photopath . $row['photo'] . '-big.jpg';
                $imagepath = $this->photopath . $row['photo'] . '-small.jpg';
                break;
        }

        ob_start();
        if ($prettypath) {

            ?>
<a href="<?php echo $prettypath ?>" rel="prettyPhoto" title="<?php echo $row['recipename'] ?>">
  <img src="<?php echo $imagepath ?>" alt="<?php echo $row['photo'] ?>" style="max-width: <?php echo intval($this->photosizes[$size]) ?>px;" width="<?php echo intval($this->photosizes[$size]) ?>"<?php echo $class ?> />
</a>
<?php
        } else {

            ?><img src="<?php echo $imagepath ?>" alt="<?php echo $row['photo'] ?>" title="<?php echo $row['recipename'] ?>" style="max-width: <?php echo intval($this->photosizes[$size]) ?>px;" width="<?php echo intval($this->photosizes[$size]) ?>"<?php echo $class ?> /><?php
        }
        $photo = ob_get_clean();
        return $photo;
    }

    protected function _sendmail_allowed()
    {
        return MX_IS_USER || MX_IS_ADMIN || $this->anonym_mail;
    }

    /**
     * pmxCaptcha::__get()
     *
     * @param string $name
     * @return
     */
    public function __get($name)
    {
        if (array_key_exists($name, $this->_config)) {
            return $this->_config[$name];
        }
        $trace = debug_backtrace();
        trigger_error('undefined property \'' . $name . '\' in ' . mx_strip_sysdirs($trace[0]['file']) . ' line ' . $trace[0]['line'], E_USER_NOTICE);
        return null;
    }

    /**
     * pmxCaptcha::__set()
     *
     * @param string $name
     * @param mixed $value
     * @return
     */
    public function __set($name, $value)
    {
        $this->_config[$name] = $value;
    }
}

?>